GUFF_PAREN_WORDS = [
    "a cappella",
    "acoustic",
    "bonus",  
    "censored",
    "clean",
    "club",                                                                     
    "clubmix",
    "composition",
    "cut",
    "dance",
    "demo",
    "dialogue",
    "dirty",                                  
    "edit",
    "excerpt",
    "explicit",
    "extended",
    "instrumental",
    "interlude",
    "intro",
    "karaoke",
    "live",
    "long",                   
    "main",
    "maxi",
    "megamix",
    "mix",
    "mono",
    "official",
    "orchestral",
    "original",
    "outro",
    "outtake",
    "outtakes",
    "piano",
    "quadraphonic",
    "radio",
    "rap",
    "re-edit",
    "reedit",
    "refix",
    "rehearsal",
    "reinterpreted",
    "released",
    "release",
    "remake",
    "remastered",
    "remaster",
    "master",  
    "remix",
    "remixed",
    "remode",
    "reprise",                                 
    "rework",
    "reworked",
    "rmx",
    "session",
    "short",
    "single",
    "skit",
    "stereo",
    "studio",
    "take",
    "takes",
    "tape",
    "track",
    "tryout",
    "uncensored",
    "unknown",
    "unplugged",
    "untitled",
    "version",
    "ver",
    "video",
    "vocal",
    "vs",                             
    "with",
    "without"
]
